package com.engine.organization.service.impl;

import com.api.browser.bean.SearchConditionGroup;
import com.api.browser.bean.SearchConditionItem;
import com.cloudstore.eccom.constant.WeaBoolAttr;
import com.cloudstore.eccom.pc.table.WeaTableColumn;
import com.cloudstore.eccom.pc.table.WeaTableOperates;
import com.cloudstore.eccom.result.WeaResultMsg;
import com.engine.common.util.ServiceUtil;
import com.engine.core.impl.Service;
import com.engine.organization.component.OrganizationWeaTable;
import com.engine.organization.entity.column.TableColumnPO;
import com.engine.organization.entity.danikor.vo.ResourceBackVO;
import com.engine.organization.mapper.danikor.DanikorResourceMapper;
import com.engine.organization.mapper.resource.HrmResourceMapper;
import com.engine.organization.service.DanikorResourceBackService;
import com.engine.organization.util.MenuBtn;
import com.engine.organization.util.OrganizationFormItemUtil;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.general.Util;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2023/01/17
 * @version: 1.0
 */
public class DanikorResourceBackServiceImpl extends Service implements DanikorResourceBackService {
    private HrmResourceMapper getHrmResourceMapper() {
        return MapperProxyFactory.getProxy(HrmResourceMapper.class);
    }

    private DanikorResourceMapper getDanikorResourceMapper() {
        return MapperProxyFactory.getProxy(DanikorResourceMapper.class);
    }


    @Override
    public Map<String, Object> listPage(Map<String, Object> params) {
        HrmResourceServiceImpl resourceService = ServiceUtil.getService(HrmResourceServiceImpl.class, user);
        OrganizationWeaTable<ResourceBackVO> table = new OrganizationWeaTable<>(user, ResourceBackVO.class);
        String syncDate = Util.null2String(params.get("syncDate"));
        String sqlForm = "from hrmresource t left join cus_fielddata t0 on t.id = t0.id and t0.scopeid ='-1' left join cus_fielddata t1 on t.id = t1.id and t1.scopeid ='1' left join cus_fielddata t2 on t.id = t2.id and t2.scopeid ='3' left join hrmjobtitles t3 on t.jobtitle=t3.id left join hrmjobactivities t4 on t3.jobactivityid=t4.id left join hrmjobgroups t5 on t4.jobgroupid=t5.id ";
        String sqlWhere = "where t.status < 4 ";
        String cusTableName = "cus_fielddata";
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            sqlWhere += " and t.sync_date = '" + syncDate + "'";
            sqlForm = "from hrmresource_back t left join cus_fielddata_back t0 on t.id = t0.id and t0.scopeid = '-1' and t.sync_date = t0.sync_date left join cus_fielddata_back t1 on t.id = t1.id and t1.scopeid = '1' and t.sync_date = t1.sync_date left join cus_fielddata_back t2 on t.id = t2.id and t2.scopeid = '3' and t.sync_date = t2.sync_date left join hrmjobtitles t3 on t.jobtitle = t3.id left join hrmjobactivities t4 on t3.jobactivityid = t4.id left join hrmjobgroups t5 on t4.jobgroupid = t5.id ";
            cusTableName = "cus_fielddata_back";
        }
        table.setSqlform(sqlForm);
        table.setSqlwhere(sqlWhere);
        table.setOperates(new WeaTableOperates());
        List<WeaTableColumn> weaTableColumnList = new ArrayList<>();
        List<String> fields = new ArrayList<>();
        List<SearchConditionGroup> allConditions = resourceService.getAllConditions();

        //初次使用，无模板初始值
        String columns = "-1_hrm_lastname,-1_hrm_departmentid,-1_hrm_subcompanyid1,-1_hrm_jobtitle,-1_hrm_mobile,-1_hrm_telephone,-1_hrm_managerid,-1_hrm_dsporder";

        List<String> cusFieldNames = getDanikorResourceMapper().getCusColumnByTableName(cusTableName).stream().map(TableColumnPO::getColumnName).collect(Collectors.toList());
        List<String> columnList = Arrays.asList(columns.split(","));
        BigDecimal decimal = new BigDecimal(100 / columnList.size());
        for (SearchConditionGroup allCondition : allConditions) {
            List<SearchConditionItem> items = allCondition.getItems();
            for (SearchConditionItem item : items) {
                String columnName = item.getDomkey()[0];
                String scopeId = columnName.split("_")[0];
                String fieldName = columnName.substring(columnName.lastIndexOf("_") + 1);
                if (columnName.contains("_cus_") && !cusFieldNames.contains(fieldName)) {
                    continue;
                }
                fields.add(resourceService.buildTableSql(columnName) + " as " + resourceService.buildTableSql(columnName).replace(".", "_"));

                WeaTableColumn weaTableColumn = new WeaTableColumn();
                weaTableColumn.setText(getHrmResourceMapper().queryLabelName(fieldName, scopeId));
                weaTableColumn.setColumn(resourceService.buildTableSql(columnName).replace(".", "_"));

                weaTableColumn.setDisplay(columnList.contains(columnName) ? WeaBoolAttr.TRUE : WeaBoolAttr.FALSE);
                if ("-1_hrm_subcompanyid1".equals(columnName)) {
                    weaTableColumn.setTransmethod("com.engine.organization.transmethod.HrmResourceTransMethod.getCompanyName");
                } else {
                    weaTableColumn.setTransmethod("com.engine.organization.transmethod.HrmResourceTransMethod.getFieldTrueValue");
                }
                weaTableColumn.setOtherpara(columnName);
                weaTableColumn.setWidth(decimal.setScale(2, RoundingMode.HALF_UP).doubleValue() + "%");
                weaTableColumnList.add(weaTableColumn);
            }
        }

        // 增加id字段,跳转人员卡片
        WeaTableColumn weaTableColumn = new WeaTableColumn();
        weaTableColumn.setColumn("id");
        weaTableColumn.setDisplay(WeaBoolAttr.FALSE);

        weaTableColumnList.add(weaTableColumn);
        fields.add("t.id");
        table.setBackfields(StringUtils.join(fields, ","));
        table.setColumns(weaTableColumnList);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        return new HashMap<>(result.getResultMap());
    }

    @Override
    public Map<String, Object> getSearchCondition() {
        Map<String, Object> apiDatas = new HashMap<>();
        List<SearchConditionGroup> addGroups = new ArrayList<>();
        List<SearchConditionItem> conditionItems = new ArrayList<>();
        SearchConditionItem syncDate = OrganizationFormItemUtil.datePickerItem(user, 2, 16, false, 2, "查询时间", "syncDate");
        conditionItems.add(syncDate);
        addGroups.add(new SearchConditionGroup("高级搜索条件", true, conditionItems));
        apiDatas.put("conditions", addGroups);
        return apiDatas;
    }

    @Override
    public Map<String, List<MenuBtn>> getHasRight() {
        Map<String, List<MenuBtn>> btnDatas = new HashMap<>();
        ArrayList<MenuBtn> rightMenuList = new ArrayList<>();
        // 显示列定制
        rightMenuList.add(MenuBtn.rightMenu_btnColumn());
        btnDatas.put("rightMenu", rightMenuList);
        return btnDatas;
    }

    @Override
    public Map<String, Object> getSummaryData(Map<String, Object> params) {
        HashMap<String, Object> returnMap = new HashMap<>();
        String companyTable = "hrmsubcompany";
        String departmentTable = "hrmdepartment";
        String resourceTable = "hrmresource";
        String syncDate = Util.null2String(params.get("syncDate"));
        if (StringUtils.isNotBlank(syncDate) && !DateUtil.getDate(new Date()).equals(syncDate)) {
            companyTable += "_back";
            departmentTable += "_back";
            resourceTable += "_back";
        } else {
            syncDate = null;
        }
        returnMap.put("subcompany", getDanikorResourceMapper().summaryCompanyData(companyTable, syncDate));
        returnMap.put("department", getDanikorResourceMapper().summaryDepartmentData(departmentTable, syncDate));
        returnMap.put("person", getDanikorResourceMapper().summaryResourceData(resourceTable, syncDate));
        return returnMap;
    }
}
